// ----------------------------------------------------------------------------
//
// Core Dynamic Library - Plugin framework for The Core Media Player
//
//       http://www.corecoded.com - http://www.corecodec.com
//   CDL AudioSource API designed to add support for audio formats
//               CDL AudioSource API Version 1.0 (100)
//
//                        (C/C++ header)
//
//  CoreCodec SDK license:
//  Copyright (c) 2001-2003 CoreCodec, Inc.
//
//    This software is provided 'as-is', without any express or implied
//    warranty.  In no event will the authors be held liable for any damages
//    arising from the use of this software.
//
//    Permission is granted to anyone to use this software for any purpose,
//    including commercial applications, and to alter it and redistribute it
//    freely, subject to the following restrictions:
//
//    1. The origin of this software must not be misrepresented; you must not
//       claim that you wrote the original software. If you use this software
//       in a product, an acknowledgment in the product documentation would be
//       appreciated but is not required.
//    2. Altered source versions must be plainly marked as such, and must not be
//       misrepresented as being the original software.
//    3. This notice may not be removed or altered from any source distribution.
//
// ----------------------------------------------------------------------------
#ifndef _CCASPLUGIN_H_
#define _CCASPLUGIN_H_
// ----------------------------------------------------------------------------

#define CCAS_API_CALL        __stdcall

// ----------------------------------------------------------------------------

#ifdef __cplusplus
#define CCAS_API_EXPORT      extern "C" __declspec(dllexport)
#else
#define CCAS_API_EXPORT      __declspec(dllexport)
#endif 

// ----------------------------------------------------------------------------

#define CCAS_PLUGIN_VERSION 100

#define CCAS_CANT_SEEK 0
#define CCAS_CAN_SEEK  1

#define CCAS_ERROR     0
#define CCAS_OK        1

typedef struct
{
	// Get the API version used for this plugin, set it to CCAS_PLUGIN_VERSION
	int   api_version;

	// The name of the plugin
	char* name;

	// Extension supported by this plugin ("mpc|mpp|mp+") if any
	char* extension;

	// Protocol supported by this plugin (ie: "file|http|ftp")
	char* protocol;

	// Open a file, return CCAS_ERROR if you can't open it, CCAS_OK if you can
	int   (CCAS_API_CALL *Open)(char* file_name);

	// Get lenght of the file in ms
	int   (CCAS_API_CALL *GetLength)(void);

	// Return CCAS_CAN_SEEK if your plugin can seek, CCAS_CANT_SEEK otherwise
	int   (CCAS_API_CALL *CanSeek)(void);

	// Seek to a position in ms
	int   (CCAS_API_CALL *Seek)(int new_pos);

	// Used for format that doesn't support seeking
	int   (CCAS_API_CALL *Restart)(void);

	// Get position in ms
	int   (CCAS_API_CALL *GetPosition)(void); 
	
	// Tell directshow the size of the buffer to use
	int   (CCAS_API_CALL *GetSuggestedBufferSize)(void);

	// Get info about the PCM format return by GetPCMData
	int   (CCAS_API_CALL *GetPCMInfo)(int* samplePerSeconds, int* bitsPerSample, int* channels);

	// Fill the buffer with a block of data, return the real length of the data
	// in the buffer
	int   (CCAS_API_CALL *GetPCMData)(char* buff, int bufflen);

	// Close the plugin
	void  (CCAS_API_CALL *Close)(void);

} AudioSourcePlugin, *PAudioSourcePlugin;

// ----------------------------------------------------------------------------
#endif // _CCASPLUGIN_H_
// ----------------------------------------------------------------------------
