/*
    Copyright 2006-2012 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt
*/

#include <stdio.h>
#include <stdlib.h>
#include <direct.h>
#include <windows.h>
#include "out.h"
#include "in2.h"
#include "dsp.h"
#include "vis.h"
#include "ml.h"
#include "gen.h"
#include "xmpdsp.h"
#include "CCASPlugin.h"



#define VER     "Winamp plug-in info 0.1.1a"
#define ADD     len += sprintf(                     \
                    buff + len,
#define GETINFO(a, b, c, d, e, f, g)                \
                typedef e *(WINAPI *b)(f);          \
                b       a;                          \
                e       *d;                         \
                a = (b)GetProcAddress(hLib, c);     \
                if(a) {                             \
                    d = a(g);                       \
                    ADD
#define GETMODULES(a)                               \
                ADD "Modules:\n");                  \
                                                    \
                modm = (getModule)a->getModule;     \
                for(i = 0; (mod = modm(i)); i++) {  \
                    ADD "    - %s\n",               \
                        mod->description);          \
                }



char *wtob(char *data);
int lame_msgbox_text(char *buff, int len, u_char *func, u_int offset);
int show_fileext(char *buff, int len, char *ext);
char *get_file(void);
void std_err(char *err);



typedef struct {
    char    *description;
    /* generic structure, the rest is not important */
} module;

typedef struct {
    HWND    xmpwin;
    char    *description;
    char    *FileExtensions;
} XMPIN;



typedef module  *(WINAPI *getModule)(int which);



int main(int argc, char *argv[]) {
    module      *mod;
    getModule   modm;
    AudioSourcePlugin   ccasx;

    HINSTANCE   hLib;
    int         i,
                len;
    char        buff[8192],
                tmpbuff[256],
                *fname,
                *fdir,
                *p,
                *w;

    fname = (argc < 2) ? get_file() : argv[1];

    fdir = strdup(fname);
    p = strrchr(fdir, '\\');
    if(p) {
        *p = 0;
        w = strrchr(fdir, '\\');
        if(w && !stricmp(w + 1, "plugins")) {
            *w = 0;
            *p = '\\';
            fname = w + 1;
        } else {
            fname = p + 1;
        }
        chdir(fdir);
    }

    hLib = LoadLibrary(fname);
    //hLib = LoadLibraryEx(fname, NULL, LOAD_WITH_ALTERED_SEARCH_PATH);
    if(!hLib) std_err(NULL);


    len = sprintf(
        buff,
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n");

    strcpy(buff + len,"\n" "unknown plug-in type\n");

    GETINFO(outm, winampGetOutModule, "winampGetOutModule", out, Out_Module,,)
            "\n"
            "OUTPUT (%d):\n"
            "\n"
            "Description\t%s\n"
            "Plug-in ID \t%d\n",
            out->version,
            out->description,
            out->id);
    }

    GETINFO(inm, winampGetInModule2, "winampGetInModule2", in, In_Module,,)
            "\n"
            "INPUT (%d):\n"
            "\n"
            "Description\t%s\n"
            "Seekable   \t%s\n"
            "Output sup.\t%s\n",
            in->version,
            in->description,
            in->is_seekable    ? "yes" : "no",
            in->UsesOutputPlug ? "yes" : "no");

        len = show_fileext(buff, len, in->FileExtensions);
    }

    GETINFO(dspm, winampDSPGetHeader2, "winampDSPGetHeader2", dsp, winampDSPHeader,,)
            "\n"
            "DSP (%d):\n"
            "\n"
            "Description\t%s\n",
            dsp->version,
            dsp->description);

        GETMODULES(dsp);
    }

    GETINFO(vism, winampVisGetHeader, "winampVisGetHeader", vis, winampVisHeader,,)
            "\n"
            "VISUALIZATION (%d):\n"
            "\n"
            "Description\t%s\n",
            vis->version,
            vis->description);

        GETMODULES(vis);
    }

    GETINFO(mlm, winampGetMediaLibraryPlugin, "winampGetMediaLibraryPlugin", ml, winampMediaLibraryPlugin,,)
            "\n"
            "MEDIA LIBRARY (%d):\n"
            "\n"
            "Description\t%s\n",
            ml->version,
            ml->description);
    }

    GETINFO(genm, winampGetGeneralPurposePlugin, "winampGetGeneralPurposePlugin", gen, winampGeneralPurposePlugin,,)
            "\n"
            "GENERAL PURPOSE (%d):\n"
            "\n"
            "Description\t%s\n",
            gen->version,
            gen->description);
    }

    typedef int (WINAPI *encplugin)(int boh, char *data);
    encplugin   encm;
    encm = (encplugin)GetProcAddress(hLib, "GetAudioTypes3");
    if(encm) {
        i = encm(0, tmpbuff);
        ADD
            "\n"
            "ENCODER (%.4s):\n"
            "\n"
            "Description\t%s\n",
            (char *)&i,
            tmpbuff);
    }

#define ARG1    int, void *
#define ARG2    1, NULL
    GETINFO(xmpinm, XMPIN_GetInterface, "XMPIN_GetInterface", xmpin, XMPIN, ARG1, ARG2)
#undef ARG1
#undef ARG2
            "\n"
            "XMPLAY INPUT:\n"
            "\n"
            "Description\t%s\n"
            "Extensions \t%s | %s\n",
            !xmpin ? "none" : xmpin->description,
            !xmpin ? "none" : xmpin->FileExtensions,
            !xmpin ? "none" : xmpin->FileExtensions + strlen(xmpin->FileExtensions) + 1);
    }

    GETINFO(xmpdspm, XMPDSP_GetInterface, "XMPDSP_GetInterface", xmpdsp, XMPDSP,,)
            "\n"
            "XMPLAY DSP (flag %u):\n"
            "\n"
            "Description\t%s\n",
            (int)xmpdsp->flags,
            xmpdsp->name);

        len = lame_msgbox_text(buff, len, (void *)xmpdsp->About, (u_int)hLib);
    }

    GETINFO(xmparcextm, XMPARC_GetExtensions, "XMPARC_GetExtensions", xmparcext, char,,)
            "\n"
            "XMPLAY ARCHIVE:\n"
            "\n"
            "Description\t%s\n",
            xmparcext);
    }

    GETINFO(ccasm, CCAS_GetPluginHeader2, "CCAS_GetPluginHeader2", ccas, PAudioSourcePlugin, PAudioSourcePlugin, &ccasx)
            "\n"
            "THE CORE MEDIA PLAYER (%d):\n"
            "\n"
            "Description\t%s\n"
            "Extensions \t%s\n"
            "Protocol   \t%s\n",
            ccasx.api_version,
            ccasx.name,
            ccasx.extension,
            ccasx.protocol);
    }

    GETINFO(musikcube_namem, GetPluginName, "GetPluginName", musikcube_name, char,,)
            "\n"
            "MUSIKCUBE:\n"
            "\n"
            "Plugin name\t%s",
            wtob(*(u_char **)(musikcube_name + 4)));
    }
    GETINFO(musikcube_descm, GetPluginDescription, "GetPluginDescription", musikcube_desc, u_char,,)
            "Description\t%s\n",
            wtob(*(u_char **)(musikcube_desc + 4)));
    }
    GETINFO(musikcube_revm, GetPluginRevision, "GetPluginRevision", musikcube_rev, int,,)
            "Revision   \t%d\n",
            (int)musikcube_rev);
    }
    GETINFO(musikcube_formm, GetPluginFormats, "GetPluginFormats", musikcube_form, char,,)
            "Formats    \t%s\n",
            wtob(*(u_char **)(musikcube_form + 4)));
    }

        /*
        I have removed any buffer-overflow check since I can do nothing
        for the encoding plug-ins which use lstrcpy() and then the code
        becomes too dirty with these checks
        */

    MessageBox(0, buff, VER, MB_OK | MB_ICONINFORMATION | MB_TASKMODAL);

    FreeLibrary(hLib);
    return(0);
}



char *wtob(char *data) {
    int         i;
    static char buff[256 + 2];

    for(i = 0; i < 256; i++) {
        if(!*data) break;
        buff[i] = *data;
        data += 2;
    }
    buff[i] = 0;
    return(buff);
}



// lame but works
int lame_msgbox_text(char *buff, int len, u_char *func, u_int offset) {
    u_char  *limit,
            *t1,
            *t2;

    t1 = t2 = "";
    offset >>= 24;  // gets the first part of the offset

    for(limit = func + 100; func < limit; func++) {
        if((func[0] == 0x68) && (func[4] == offset)) {
            t1 = *(u_char **)(func + 1);
            break;
        }
    }

    for(func += 5; func < limit; func++) {
        if((func[0] == 0x68) && (func[4] == offset)) {
            t2 = *(u_char **)(func + 1);
            break;
        }
    }

    ADD
        "More Info  \t%s\n"
        "Plug-in comment / About:\n"
        "%s\n",
        t1,
        t2);

    return(len);
}



int show_fileext(char *buff, int len, char *ext) {
    char    *l;

    while(*ext) {
        l = ext + strlen(ext) + 1;
        ADD
            "Extensions \t%s %s\n",
            ext, l);
        ext = l + strlen(l) + 1;
    }

    return(len);
}



char *get_file(void) {
    OPENFILENAME    ofn;
    static char     filename[MAX_PATH + FILENAME_MAX + 1];

    memset(&ofn, 0, sizeof(ofn));
    ofn.lStructSize  = sizeof(ofn);
    ofn.lpstrFilter  = "*.dll;*.cdl\0"  "*.dll;*.cdl\0"
                       "*.*\0"          "*.*\0";
    ofn.nFilterIndex = 1;
    ofn.lpstrFile    = filename;
    ofn.nMaxFile     = sizeof(filename) - 1;
    ofn.lpstrTitle   = VER;
    ofn.Flags        = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_LONGNAMES | OFN_EXPLORER;

    if(!GetOpenFileName(&ofn)) {
        //std_err("Error while creating the dialog box for the selection of the file or probably you have selected no file");
        exit(1);
    }

    return(filename);

    filename[ofn.nFileOffset - 1] = 0;
    chdir(filename);
    return(filename + ofn.nFileOffset);
}



void std_err(char *err) {
    if(!err) {
        FormatMessage(
          FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
          NULL,
          GetLastError(),
          0,
          (char *)&err,
          0,
          NULL);
    }
    MessageBox(0, err, VER, MB_OK | MB_ICONERROR | MB_TASKMODAL);
    exit(1);
}


